;################################################################################
;#										#
;# Z80 emulator		 							#
;# copyright (c) 2009-2011 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;interrupt
;------------------------------------------------------------------------------
emu_int:	sbiw	PC_L,1			;2 discard current byte
		ldi	ZH,HIGH(emu_jtab)	;1 next cycle is normal
		out	EEARL,ZH

		cbi	GPIOR0,int_flag
		lds	r16,z80_iflag
		sbrs	r16,0
		jmp	emu_loop		;int is disabled, repeat last cmd
		sts	z80_iflag,const_0	;disable further INT
		lds	r16,z80_imode
		andi	r16,0x03
		cpi	r16,2
		brne	emu_int_mx
		lds	XH,zreg_i
		z80_intvec			;get int vector
		getmembyte r0,XL,XH
		adiw	XL,1
		getmembyte r1,XL,XH
		jmp	emu_intcall		;this inserts a call

emu_int_mx:	cpi	r16,1
		brne	emu_int_m0
		ldi	r16,0x38
		jmp	emu_rst			;do a RST38

emu_int_m0:	z80_intvec
		mov	ZL,XL
		in	ZH,EEARL
		ijmp

;------------------------------------------------------------------------------
; the level1 jump table for interrupt
;------------------------------------------------------------------------------

.org (pc+255) & 0xff00

emu_jitab:	rjmp	emu_int			;code 0x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int

		rjmp	emu_int			;code 1x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code 2x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code 3x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int

		rjmp	emu_int			;code 4x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code 5x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code 6x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code 7x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code 8x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code 9x
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code ax
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code bx
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code cx
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code dx
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code ex
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int


		rjmp	emu_int			;code fx
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int
		rjmp	emu_int

