;################################################################################
;#										#
;# Z80 emulator		 							#
;# copyright (c) 2009-2011 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################
.if rom_tables == 0
emu_copytables:	;copy flag tables
		push	ZH
		push	ZL
		push	YH
		push	YL
		push	XH
		push	XL
		ldi	ZL,LOW(emu_flagtab_l*2)
		ldi	ZH,high(emu_flagtab_l*2)
		ldi	YL,LOW(emu_flags_log)
		ldi	YH,HIGH(emu_flags_log)
		clr	XH
emu_cpt1:	elpm	XL,Z+
		st	Y+,XL
		elpm	XL,Z+
		st	Y+,XL
		dec	XH
		brne	emu_cpt1
		pop	XL
		pop	XH
		pop	YL
		pop	YH
		pop	ZL
		pop	ZH
		ret
.endif

emu_copy_mem:	;copy memory data
		push	ZH
		push	ZL
		push	YH
		push	YL
		push	XH
		push	XL
		ldi	ZH,0x3e			;user map
		lds	XL,emu_umode
		cpi	XL,0x01
		breq	emu_cpm05
		ldi	ZH,0x3f			;switch to default map
		cpi	XL,0x00
		breq	emu_cpm05
		ldi	ZH,high(mmap_sys*2)	;default map
emu_cpm05:	ldi	ZL,0x00
		ldi	YL,LOW(emu_memtab)
		ldi	YH,HIGH(emu_memtab)
		push	ZH

		ldi	XH,0x40			;ROM
emu_cpm1:	elpm	XL,Z+
		ldi	XL,0xff
		st	Y+,XL
		dec	XH
		brne	emu_cpm1

		ldi	XH,0xc0			;ROM/RAM
emu_cpm3:	elpm	XL,Z+
		st	Y+,XL
		dec	XH
		brne	emu_cpm3

		pop	ZH
		clr	ZL

		;get system data
		elpm	PC_L,Z+
		elpm	PC_H,Z+
		elpm	SP_L,Z+
		elpm	SP_H,Z+

		elpm	reg_f,Z+
		elpm	reg_a,Z+
		elpm	reg_c,Z+
		elpm	reg_b,Z+
		elpm	reg_e,Z+
		elpm	reg_d,Z+
		elpm	reg_l,Z+
		elpm	reg_h,Z+

		elpm	XL,Z+
		in	XH,GPIOR0
		andi	XH,0xf0
		andi	XL,0x0f
		or	XL,XH
		out	GPIOR0,XL

		elpm	XL,Z+
		mov	reg_border,XL
		elpm	XL,Z+
		sts	vsys_sborder,XL

		ldi	YL,LOW(z80emu_ram)
		ldi	YH,HIGH(z80emu_ram)
		ldi	XH,34
emu_cpm_4:	elpm	XL,Z+
		st	Y+,XL
		dec	XH
		brne	emu_cpm_4

		pop	XL
		pop	XH
		pop	YL
		pop	YH
		pop	ZL
		pop	ZH
		ret

emu_rcopy:	;copy ROM to RAM if necessary (default mode)
		lds	YL,emu_umode
		cpi	YL,0
		brne	emu_rcopy_u		;-> user mode
		ldi	YL,LOW(emu_memtab)
		ldi	YH,HIGH(emu_memtab)
		ldi	r16,0x00		;page num

emu_rcopy_d1:	ld	XH,Y			;get table entry
		sbrc	XH,7			;skip if RAM
		rjmp	emu_rcopy_d4		;ROM -> IGNORE PAGE

		add	r16,const_1		;reduce free pages
		cpi	r16,max_pages+1		;beyond last
		brcc	emu_rcopy_d6		;no page available (error)
		mov	ZH,YL			;ROM address high = page number
		clr	ZL			;ROM page start
		clr	XL			;RAM page start

		ldi	r17,0			;copy 256 bytes
emu_rcopy_d2:	lpm	r18,Z+			;copy ROM data...
		st	X+,r18			;to RAM
		dec	r17
		brne	emu_rcopy_d2

emu_rcopy_d4:	inc	YL
		cpi	YL,0x00
		brne	emu_rcopy_d1
		sts	emu_ram_mpage,r16
		out	RAMPZ,const_1
		ret

emu_rcopy_d6:	ldi	XL,0x40
		mov	reg_border,XL
emu_rcopy_d7:	rjmp	emu_rcopy_d7


emu_rcopy_u:	;copy ROM to RAM (user mode)
		cpi	YL,1
		brne	emu_rcopy_s		;-> sys mode

		ldi	YL,LOW(emu_memtab)
		ldi	YH,HIGH(emu_memtab)
		ldi	r16,0x00		;free pages (15K)

emu_rcopy_u1:	ld	XH,Y			;get table entry
		sbrc	XH,7			;skip if RAM
		rjmp	emu_rcopy_u4		;ROM -> IGNORE PAGE

		add	r16,const_1		;reduce free pages
		cpi	r16,max_pages+1		;beyond last
		brcc	emu_rcopy_u6		;no page available (error)
		mov	ZH,XH			;ROM address high
		dec	ZH
		clr	ZL			;ROM page start
		clr	XL			;RAM page start

		ldi	r17,0			;copy 256 bytes
emu_rcopy_u2:	elpm	r18,Z+			;copy ROM data...
		st	X+,r18			;to RAM
		dec	r17
		brne	emu_rcopy_u2

emu_rcopy_u4:	inc	YL
		cpi	YL,0x00
		brne	emu_rcopy_u1
		sts	emu_ram_mpage,r16
		out	RAMPZ,const_1
		ret

emu_rcopy_u6:	ldi	XL,0x40
		mov	reg_border,XL
emu_rcopy_u7:	rjmp	emu_rcopy_u7


emu_rcopy_s:	movw	YL,const_0
		ldi	XL,0xff
emu_rcopy_s1:	st	Y+,XL
		cpi	YH,0x3f
		brne	emu_rcopy_s1
		subi	YH,0x02
		sts	emu_ram_mpage,YH
		out	RAMPZ,const_1
		ret


;------------------------------------------------------------------------------
;init registers and start
;------------------------------------------------------------------------------
emu_start_usr:	sts	emu_umode,const_1
		rjmp	emu_start_2

emu_start_sys:	ldi	XL,2
		sts	emu_umode,XL
		rjmp	emu_start_2

emu_start:	sts	emu_umode,const_0		;use default
		;copy flag tables to RAM
emu_start_2:
.if rom_tables == 0
		rcall	emu_copytables
.endif

		;copy default memory table to RAM
		rcall	emu_copy_mem

		;copy ROM to RAM if necessary
		rcall	emu_rcopy

emu_start_4:	ldi	ZH,HIGH(emu_jtab)	;1
		out	EEARL,ZH

		set_refresh			;initial R register value

;ppo:		rjmp	ppo

		rjmp	emu_loop
