;################################################################################
;#										#
;# AX82 computer with ATmega644/1284						#
;# copyright (c) 2011-2012 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License as		#
;# published by the Free Software Foundation; either version 3 of		#
;# the License, or (at your option) any later version.				#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;--------------------------------------------------------------------------------
; common definitions
;--------------------------------------------------------------------------------
.set	use_refresh = 0			;increment R register
.set	rom_tables = 1
.set	use_example = 1

.def	const_0 = r2
.def	const_1 = r3
.def	const_2 = r12
.def	reg_border = r13
.def	sreg_save = r14
.def	const_ff = r15

.include "/usr/local/include/avr/m1284Pdef.inc"
.equ		emu_ram		= 0x0100	;start of RAM (and dfile)

.if rom_tables == 0
	.equ		emu_memtab	= 0x3d00	;memory switch table
	.equ		emu_flags_log	= 0x3e00	;logic flags (RAM copy)
	.equ		emu_regtab	= 0x3e00	;memory switch table
	.equ		emu_flags_rot	= 0x3f00	;rotation flags (RAM copy)
	.equ		max_pages	= 0x3c		;max page
.else
	.equ		emu_memtab	= 0x3f00	;memory switch table
	.equ		max_pages	= 0x3e		;max page
.endif

.equ		fsys_buffer	= 0x3000	;filesystem buffer
.equ		vsys_kbuf	= 0x4000	;16 bytes keyboard buffer
.equ		vsys_ram	= 0x4010	;64 bytes vsys system RAM
.equ		z80emu_ram	= 0x4050	;48 bytes shadow register etc.
.equ		fsys_ram	= 0x4080	;32 bytes filesystem RAM
.equ		stack		= 0x40fd	;stack

.include	"config/pin_config.inc"

;--------------------------------------------------------------------------------
; module specific definitions
;--------------------------------------------------------------------------------
.include	"emulator/definitions.inc"
.include	"vsys/definitions.inc"
.include	"vsys/timing.inc"
.include	"modules/definitions.inc"

;--------------------------------------------------------------------------------
; macro definitions
;--------------------------------------------------------------------------------
.include	"emulator/mem_macros.asm"
.include	"emulator/emu_macros.asm"
.include	"emulator/io_macros.asm"

.CSEG
.org 0x0000

;--------------------------------------------------------------------------------
; application-specific code
;--------------------------------------------------------------------------------
romdata:
.include	"tables/romdata_spectrum.inc"
.if use_example == 0
.include	"tables/testdat.inc"
.include	"tables/testdat.inc"
.include	"tables/testdat.inc"
.include	"tables/atab.inc"

.org 0x9f00
mmap_2:
.include	"tables/default_mem.inc"

.org 0x9f80
mmap_1:
.include	"tables/default_mem.inc"
.else
	.equ	mmap_1	= 0x9f80
	.include	"example/example.inc"
.endif

;--------------------------------------------------------------------------------
; the automatic generated video out code
;--------------------------------------------------------------------------------
.org 0xa000
.include	"vsys/vout.asm"

;--------------------------------------------------------------------------------
; some tables
;--------------------------------------------------------------------------------
.org (pc+127) & 0xff80
vsys_ktables:
.include	"tables/keytable_de.inc"

.org (pc+127) & 0xff80
vsys_kstab:
.include	"tables/kstable.inc"

.org (pc+127) & 0xff80
mmap_sys:
.include	"tables/default_mem.inc"

attr_tab:
.org (pc+127) & 0xff80
.include	"tables/atab.inc"
.org (pc+127) & 0xff80
.include	"emulator/flags.inc"

.org (pc+127) & 0xff80
.include	"tables/flagconv.inc"

.org (pc+127) & 0xff80
.include	"tables/pktab.inc"
.org (pc+127) & 0xff80
.include	"tables/vad_table.inc"


;--------------------------------------------------------------------------------
; here starts the code
;--------------------------------------------------------------------------------
start:		cli				;disable video after jmp 0
		ldi	XL,0x00
		mov	const_0,XL
		ldi	XL,0x01
		mov	const_1,XL
		ldi	XL,0x02
		mov	const_2,XL
		ldi	XL,0xff
		mov	const_ff,XL

		in	XL,MCUCR
		andi	XL,0xfc
		mov	XH,XL
		ori	XL,0x01
		ori	XH,0x02
		out	MCUCR,XL
		out	MCUCR,XH

		ldi	XL,HIGH(stack)
		out	SPH,XL
		ldi	XL,LOW(stack)
		out	SPL,XL
		out	SPCR,const_0		;disable SPI
		out	RAMPZ,const_1

		call	vsys_init		;activate video
		sei

		call	intro			;show intro

loop:		lds	XL,vsys_kbdstate
		sbrc	XL,0
		jmp	emu_start		;start emulator in normal mode if left shift key is pressed
		sbrs	XL,4
		jmp	emu_start_usr		;start default in user mode
		jmp	emu_start_sys		;start in system mode if alt key is pressed

;--------------------------------------------------------------------------------
; the emulator engine
;--------------------------------------------------------------------------------
.include	"emulator/emulator_start.asm"
.include	"emulator/emulator_main.asm"
.include	"emulator/emulator_cb.asm"
.include	"emulator/emulator_xd.asm"
.include	"emulator/emulator_xdcb.asm"
.include	"emulator/emulator_ed.asm"
.include	"emulator/emulator_int.asm"
.include	"emulator/emulator_res.asm"
.include	"emulator/emulator_page.asm"


;--------------------------------------------------------------------------------
; misc. modules
;--------------------------------------------------------------------------------
.include	"modules/fsys.asm"
.include	"modules/llist.asm"
.include	"modules/intro.asm"

;--------------------------------------------------------------------------------
; video system / keyboard
;--------------------------------------------------------------------------------
.include	"vsys/init.asm"
.include	"vsys/func.asm"




.org 0xf800
kmatrix_spec:
.include	"tables/kmatrix_spec.inc"

;--------------------------------------------------------------------------------
; bootvector
;--------------------------------------------------------------------------------
.org 0xfc00
		jmp	start

.org 0xfc00 + OC2Baddr
		out	PORTC,reg_border	;border on
		reti

.org 0xfc00 + OVF2addr
		out	PORTC,const_0		;border off
		reti

.org 0xfc00 + OC1Baddr
		jmp	vint_vstart		;video interrupt VGA/TV

.include	"modules/memory.asm"

.include	"vsys/substitution.asm"
.include	"vsys/vint_frame.asm"
.include	"vsys/vint_main.asm"
.include	"vsys/vint_keyboard.asm"

