;################################################################################
;#										#
;# fsys - file system library							#
;# copyright (c) 2011-2013 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU  General Public				#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU  General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
; init filesystem and write messages
;------------------------------------------------------------------------------
fsys_init:	ldi	XL,0
		ldi	XH,0
		out	SPCR,const_0
		sbi	SPI_SPORT,SPI_SPIN	;set port pins
		sbi	SPI_SDDR,SPI_SPIN
		sbi	MOSI_DDR,MOSI_PIN
		sbi	SCK_DDR,SCK_PIN
		sts	fsys_filenr,const_0	;start with tape 0
		sts	fsys_status,const_0	;card not available
		sts	fsys_tapenr,const_0	;card not available

fsys_init_1:	nop
		nop
		nop
		sbiw	XL,1
		brne	fsys_init_1

		rcall	sdfs_init		;check for SD card

		rcall	fsys_pstat		;view type
		lds	XL,fsys_status		;status
		sbrs	XL,7
		ret				;no format check

		rcall	fsys_nfind		;find image (new version)
		rcall	fsys_fstat		;format status
		ret


;------------------------------------------------------------------------------
; wait for SPI is ready for next data
; write XL to SPI
;------------------------------------------------------------------------------
fsys_write_ff2:	rcall	fsys_write_ff
fsys_write_ff:	ldi	XL,0xff			;dummy data
fsys_wspi:	out	SPDR,XL
fsys_wait:	in	XL,SPSR
		sbrs	XL,SPIF
		rjmp	fsys_wait
		in	XL,SPDR
		ret

fsys_write_00:	clr	XL
		rjmp	fsys_wspi


;------------------------------------------------------------------------------
; print out filesystem status
;------------------------------------------------------------------------------
fsys_pstat:	lds	XL,fsys_status
		ldi	ZL,LOW(fsys_txt1*2)
		ldi	ZH,HIGH(fsys_txt1*2)
		cpi	XL,0x01			;card error
		breq	fsys_ppstat
		adiw	ZL,12
		cpi	XL,0x81			;MMC
		breq	fsys_ppstat
		adiw	ZL,12
		cpi	XL,0x82			;SDV1
		breq	fsys_ppstat
		adiw	ZL,12
		cpi	XL,0x83			;SDV2
		breq	fsys_ppstat
		adiw	ZL,12
		cpi	XL,0x84			;SDHC
		breq	fsys_ppstat
		adiw	ZL,12
		cpi	XL,0x02			;ACMD41 ERR
		breq	fsys_ppstat
		adiw	ZL,12
		cpi	XL,0x03			;CMD1 ERR
		breq	fsys_ppstat
		adiw	ZL,12
		cpi	XL,0x04			;PATTERN ERROR
		breq	fsys_ppstat
		adiw	ZL,12
fsys_ppstat:	jmp	sys_text

fsys_txt1:	.db "CARD ERROR ",0x00		;CARD ERR
		.db "MMC-CARD   ",0x00		;MMC-CARD
		.db "SDV1-CARD  ",0x00		;SDV1-CARD
		.db "SDV2-CARD  ",0x00		;SDV2-CARD
		.db "SDHC-CARD  ",0x00		;SDHC-CARD
		.db "ERR-ACMD41 ",0x00		;E-ACMD41
		.db "ERR-CMD1   ",0x00		;E-CMD1
		.db "ERR-CMD8   ",0x00		;E-CMD8
		.db "NO CARD    ",0x00		;NO CARD


;------------------------------------------------------------------------------
; print out filesystem status2
;------------------------------------------------------------------------------
fsys_fstat:	lds	XL,fsys_status
		ldi	ZL,LOW(fsys_txt2*2)
		ldi	ZH,HIGH(fsys_txt2*2)
		sbrs	XL,7
		adiw	ZL,20
		jmp	sys_text

fsys_txt2:	.db "-> AX82B IMAGE    ",0,0	;AX-IMAGE
		.db "-> NO IMAGE FOUND ",0,0	;NO IMAGE

;------------------------------------------------------------------------------
; write command XH to SPI
; XL=return byte (FF=time out)
;------------------------------------------------------------------------------
sdfs_cmd0:	clr	r16			;clear address field
		clr	r17
		movw	r18,r16

sdfs_cmd:	cbi	SPI_SPORT,SPI_SPIN	;CS active
		rcall	fsys_write_ff2		;16 dummy clocks
		andi	XH,0x3f			;limit
		ori	XH,0x40			;set bit 6
		out	SPDR,XH			;CMD
		andi	XH,0x3f			;clear bit 6
		rcall	fsys_wait
		out	SPDR,r19		;address MSB
		rcall	fsys_wait
		out	SPDR,r18
		rcall	fsys_wait
		out	SPDR,r17
		rcall	fsys_wait
		out	SPDR,r16		;address LSB
		rcall	fsys_wait
		ldi	XL,0x95			;dummy checksum
		cpi	XH,0x08
		brne	sdfs_cmd_01
		ldi	XL,0x87
sdfs_cmd_01:	rcall	fsys_wspi
		clr	r0

sdfs_cmd_resp:	rcall	fsys_write_ff		;write 0xff and read data
		cpi	XL,0xff
		brne	sdfs_cmd_end
		dec	r0
		brne	sdfs_cmd_resp
sdfs_cmd_end:	ret

;------------------------------------------------------------------------------
; wait for start block mtoken
;------------------------------------------------------------------------------
fsys_waitsbt:	push	ZL
		push	ZH
		ldi	ZL,0			;timeout
		ldi	ZH,0

fsys_waitsbt1:	cpi	XL,0xfe			;start block token
		brne	fsys_waitsbt2
		pop	ZH
		pop	ZL
		ret				;SBT found

fsys_waitsbt2:	sbiw	ZL,1			;timeout counter
		breq	fsys_waitsbt3		;branch if timeout is reached
		rcall	fsys_write_ff
		rjmp	fsys_waitsbt1		;next try

fsys_waitsbt3:	pop	ZH
		pop	ZL
		ret

;------------------------------------------------------------------------------
; init SD card and determine type
;------------------------------------------------------------------------------
sdfs_init:	;init SPI to f/128
		ldi	XL,0x53			;f/128
		out	SPCR,XL
		ldi	XL,0x04			;timeout factor

		;80 dummy pulses with CS=HI
sdfs_init_00:	ldi	XH,10
		sbi	SPI_SPORT,SPI_SPIN	;CS inactive
sdfs_init_01:	rcall	fsys_write_ff		;128 clock pulses
		dec	XH
		brne	sdfs_init_01

		ldi	YL,0x08			;CMD0 retries
		ldi	YH,0x00

sdfs_init_02:	ldi	XH,0			;CMD0
		rcall	sdfs_cmd0		;send CMD0
		cpi	XL,0xff
		breq	sdfs_init_02a
		sbrc	XL,0			;skip if idle state
		rjmp	sdfs_init_03		;yes->OK
sdfs_init_02a:	sbiw	YL,1			;retry counter
		brne	sdfs_init_02		;retry CMD 0

sdfs_init_err:	sts	fsys_status,const_0	;card not available
		sbi	SPI_SPORT,SPI_SPIN	;CS inactive
		ret				;exit

		;check for SDv2 with CMD8
sdfs_init_03:	ldi	r16,0xaa		;test pattern
		ldi	r17,0x01
		clr	r18
		clr	r19
		ldi	XH,8			;CMD8
		rcall	sdfs_cmd		;send CMD8
		sbrc	XL,2			;illegal?
		rjmp	sdfs_init_04		;yes
		rcall	fsys_write_ff2
		rcall	fsys_write_ff2
		cpi	XL,0xaa			;test pattern
		brne	sdfs_init_err1		;fail
		ldi	XL,0x90			;SDv2
		sts	fsys_status,XL		;write status
		rjmp	sdfs_init_05		;goto wait idle

		;check for MMC / SD
sdfs_init_04:	ldi	XH,55			;CMD55
		rcall	sdfs_cmd0		;send CMD55
		ldi	XH,1			;ACMD41
		rcall	sdfs_cmd0		;send ACMD41
		sbrc	XL,2			;illegal?
		rjmp	sdfs_init_20		;MMC
		ldi	XL,0x82			;SDv1
		sts	fsys_status,XL		;write status

		;wait for card ready
sdfs_init_05:	ldi	YL,0
		ldi	YH,0x80

sdfs_init_06:	ldi	XH,55			;CMD55
		rcall	sdfs_cmd0		;send CMD55
		lds	XL,fsys_status		;get stat
		sbrc	XL,4			;check V2
		ldi	r19,0x40
		ldi	XH,1			;ACMD41
		rcall	sdfs_cmd		;send ACMD41
		sbrs	XL,0			;skip if idle state
		rjmp	sdfs_init_10		;yes->OK
		sbiw	YL,1			;loops
		brne	sdfs_init_06

sdfs_init_07:	ldi	r16,0x02		;ACMD41 err
		sts	fsys_status,r16
		sbi	SPI_SPORT,SPI_SPIN	;CS inactive
		ret

sdfs_init_err1:	ldi	XL,0x04
		sts	fsys_status,XL		;card error
		sbi	SPI_SPORT,SPI_SPIN	;CS inactive
		ret				;exit

sdfs_init_10:	lds	XL,fsys_status		;get stat
		sbrs	XL,4			;
		rjmp	sdfs_init_end		;SDv1 card
		ldi	XL,0x83			;set SDv2
		sts	fsys_status,XL		;write status

		;now check for SDHC or V2
		ldi	XH,58			;CMD58
		rcall	sdfs_cmd0		;send CMD58
		rcall	fsys_write_ff
		push	XL
		rcall	fsys_write_ff
		rcall	fsys_write_ff
		rcall	fsys_write_ff
		pop	XL
		sbrs	XL,6
		rjmp	sdfs_init_end		;is SDv2 card
		ldi	XL,0x84			;SDHC card
		sts	fsys_status,XL		;write status

sdfs_init_end:	clr	r16			;512 Bytes block size
		ldi	r17,2
		clr	r18
		clr	r19
		ldi	XH,16			;CMD16
		rcall	sdfs_cmd		;send CMD16
		sbi	SPI_SPORT,SPI_SPIN	;CS inactive
		ldi	XL,0x50			;f/4
		out	SPCR,XL
		ret				;exit

sdfs_init_20:	ldi	YL,0
		ldi	YH,0x80

sdfs_init_21:	ldi	XH,1			;CMD1
		rcall	sdfs_cmd0		;send CMD1
		sbrs	XL,0			;skip if idle state
		rjmp	sdfs_init_22		;yes->OK
		sbiw	YL,1			;loops
		brne	sdfs_init_21		
		ldi	r16,0x03		;CMD1 err
		sts	fsys_status,r16
		sbi	SPI_SPORT,SPI_SPIN	;CS inactive
		ret

sdfs_init_22:	ldi	XL,0x81			;MMC card
		sts	fsys_status,XL		;write status
		rjmp	sdfs_init_end



;------------------------------------------------------------------------------
; calc sector address to r16..r19
; fsys_tapenr	= tape
; fsys_filenr	= file
; fsys_sectnr	= sector
;------------------------------------------------------------------------------
fsys_sectaddr0:	sts	fsys_sectnr,const_0	;first sector
fsys_sectaddr:	lds	XL,fsys_sectnr
		lds	XH,fsys_offset_l
		add	XL,XH
		sts	fsys_sectnr0,XL
		lds	XL,fsys_offset_m
		adc	XL,const_0
		sts	fsys_sectnr1,XL
		lds	XL,fsys_offset_h
		adc	XL,const_0
		sts	fsys_sectnr2,XL
		lds	XL,fsys_offset_e
		adc	XL,const_0
		sts	fsys_sectnr3,XL

		lds	YL,fsys_tapenr
		ldi	ZL,LOW(4096)
		mul	YL,ZL
		movw	XL,r0
		ldi	ZL,HIGH(4096)
		mul	YL,ZL
		clr	YL
		clr	YH
		add	XH,r0
		adc	YL,r1
		adc	YH,const_0
		ldi	ZL,128
		lds	r0,fsys_filenr
		mul	ZL,r0
		add	XL,r0
		adc	XH,r1
		adc	YL,const_0
		adc	YH,const_1
		rcall	fsys_addsec
		ret

;------------------------------------------------------------------------------
; read sector
; fsys_filenr	= file
; fsys_sectnr	= sector
;------------------------------------------------------------------------------
fsys_rsector:	rcall	fsys_sectaddr		;calc sector addr
		rjmp	fsys_sread

;------------------------------------------------------------------------------
; write sector
; fsys_filenr	= file
; fsys_sectnr	= sector
;------------------------------------------------------------------------------
fsys_wsector:	rcall	fsys_sectaddr		;calc sector addr
		rjmp	fsys_swrite

;------------------------------------------------------------------------------
; read file
;------------------------------------------------------------------------------
fsys_readfile:	sts	fsys_sectnr,const_0	;start with sector 0
		sts	fsys_mwp_l,const_0
		ldi	XL,0x40			;start address
		sts	fsys_mwp_h,XL
		sbi	GPIOR0,noscreen_flag
fsys_rfile_1:	rcall	fsys_rsector		;read sector to fsys_buffer
		call	mem_writeblock		;write block to flash
		lds	XH,fsys_mwp_h
		subi	XH,0xfe			;+2
		sts	fsys_mwp_h,XH
		lds	XH,fsys_sectnr
		inc	XH
		sts	fsys_sectnr,XH
		cpi	XH,0x80
		brcs	fsys_rfile_1
		sts	vsys_keycode,const_0
		sts	vsys_kbdstate,const_0
fsys_rfile_2:	lds	XL,UCSR0A
		sbrs	XL,RXC0
		rjmp	fsys_rfile_3
		lds	XL,UDR0
		rjmp	fsys_rfile_2
fsys_rfile_3:	cbi	GPIOR0,noscreen_flag
		ldi	XL,0x20
		mov	reg_border,XL
		ret

;------------------------------------------------------------------------------
; read screen
;------------------------------------------------------------------------------
fsys_readscr:	sts	fsys_sectnr,const_0	;start with sector 0
		sts	fsys_mwp_l,const_0	;start with vram address
		sts	fsys_mwp_h,const_1
fsys_rscreen_1:	rcall	fsys_rsector
		lds	YL,fsys_mwp_l
		lds	YH,fsys_mwp_h
		ldi	XL,LOW(fsys_buffer)
		ldi	XH,HIGH(fsys_buffer)
fsys_rscreen_2:	ld	r0,X+
		st	Y+,r0
		cpi	XH,HIGH(fsys_buffer)+2
		brne	fsys_rscreen_2
		sts	fsys_mwp_l,YL
		sts	fsys_mwp_h,YH
		lds	XH,fsys_sectnr
		inc	XH
		sts	fsys_sectnr,XH
		cpi	XH,14
		brne	fsys_rscreen_1
		ret


;------------------------------------------------------------------------------
; write file
;------------------------------------------------------------------------------
fsys_writefile:	sts	fsys_sectnr,const_0	;start with sector 0
		ldi	XL,0x80
		mov	reg_border,XL

fsys_wfile_1:	lds	ZH,fsys_sectnr
		lsl	ZH
		subi	ZH,0xc0
		ldi	ZL,0
		ldi	YL,LOW(fsys_buffer)
		ldi	YH,HIGH(fsys_buffer)
fsys_wfile_2:	lpm	r0,Z+
		st	Y+,r0
		cpi	YH,HIGH(fsys_buffer)+2
		brne	fsys_wfile_2
		rcall	fsys_wsector
		lds	XH,fsys_sectnr
		inc	XH
		sts	fsys_sectnr,XH
		cpi	XH,96
		brne	fsys_wfile_1
		out	RAMPZ,const_1

fsys_wfile_3:	lds	ZH,fsys_sectnr
		subi	ZH,96
		lsl	ZH
		ldi	ZL,0
		ldi	YL,LOW(fsys_buffer)
		ldi	YH,HIGH(fsys_buffer)
fsys_wfile_4:	elpm	r0,Z+
		st	Y+,r0
		cpi	YH,HIGH(fsys_buffer)+2
		brne	fsys_wfile_4
		rcall	fsys_wsector
		lds	XH,fsys_sectnr
		inc	XH
		sts	fsys_sectnr,XH
		cpi	XH,127
		brne	fsys_wfile_3

fsys_wfile_5:	lds	ZH,fsys_sectnr
		subi	ZH,96
		lsl	ZH
		ldi	ZL,0
		ldi	YL,LOW(fsys_buffer)
		ldi	YH,HIGH(fsys_buffer)
fsys_wfile_6:	elpm	r0,Z+
		st	Y+,r0
		cpi	YH,HIGH(fsys_buffer)+2
		brne	fsys_wfile_6
		ldi	YL,LOW(fsys_buffer+35)
		ldi	YH,HIGH(fsys_buffer+35)
		ldi	XL,LOW(fsys_buf2)
		ldi	XH,HIGH(fsys_buf2)
		ldi	ZH,12
fsys_wfile_7:	ld	ZL,X+
		st	Y+,ZL
		dec	ZH
		brne	fsys_wfile_7
		st	Y+,const_0
		ldi	XL,0x20
		mov	reg_border,XL
		rjmp	fsys_wsector


;-----------------------------------------------------------------------------
; find image file
;-----------------------------------------------------------------------------
fsys_nfind:	clr	r16
		rcall	fsys_rdmbr		;read MBE
		cpi	r16,0x00
		brne	fsys_nfind_err
		rcall	fsys_rbs		;read Bootsector
		cpi	r16,0x00
		brne	fsys_nfind_err
		rcall	fsys_rdir		;read root directory
		cpi	r16,0x00
		brne	fsys_nfind_err
		ret

fsys_nfind_err:	mov	XH,r16
		call	sys_hexout
		sts	fsys_status,const_0
		ret


;-----------------------------------------------------------------------------
; read sector
;-----------------------------------------------------------------------------
fsys_sread:	cbi	SPI_SPORT,SPI_SPIN	;activate
		rcall	fsys_write_ff2		;16 dummy clocks
		ldi	XL,0x51			;read (17) + 64
		rcall	fsys_wspi
		lds	XL,fsys_status
		sbrs	XL,2
		rjmp	fsys_sread_10		;no SDHC
		lds	XL,fsys_sectnr3
		rcall	fsys_wspi
		lds	XL,fsys_sectnr2
		rcall	fsys_wspi
		lds	XL,fsys_sectnr1
		rcall	fsys_wspi
		lds	XL,fsys_sectnr0
		rcall	fsys_wspi
fsys_sread_1:	ldi	XL,0x95			;dummy crc
		rcall	fsys_wspi
		clr	r0
fsys_sread_2:	rcall	fsys_write_ff
		cpi	XL,0xff
		brne	fsys_sread_3
		dec	r0
		brne	fsys_sread_2
fsys_sread_3:	rcall	fsys_waitsbt		;wait for start block token
		ldi	XH,0x00
		ldi	YL,LOW(fsys_buffer)
		ldi	YH,HIGH(fsys_buffer)
fsys_sread_4:	rcall	fsys_write_ff
		st	Y+,XL
		rcall	fsys_write_ff
		st	Y+,XL
		dec	XH
		brne	fsys_sread_4
		rcall	fsys_write_ff2
		sbi	SPI_SPORT,SPI_SPIN	;de-activate
		ret

fsys_sread_10:	lds	XL,fsys_sectnr2
		lds	XH,fsys_sectnr1
		lsl	XH
		rol	XL
		rcall	fsys_wspi
		mov	XL,XH
		lsr	XL
		lds	XH,fsys_sectnr0
		lsl	XH
		rol	XL
		rcall	fsys_wspi
		mov	XL,XH
		rcall	fsys_wspi
		clr	XL
		rcall	fsys_wspi
		rjmp	fsys_sread_1

;-----------------------------------------------------------------------------
; write sector
;-----------------------------------------------------------------------------
fsys_swrite:	cbi	SPI_SPORT,SPI_SPIN	;activate
		rcall	fsys_write_ff2		;16 dummy clocks
		ldi	XL,0x58			;write (24) + 64
		rcall	fsys_wspi
		lds	XL,fsys_status
		sbrs	XL,2
		rjmp	fsys_swrite_10		;no SDHC
		lds	XL,fsys_sectnr3
		rcall	fsys_wspi
		lds	XL,fsys_sectnr2
		rcall	fsys_wspi
		lds	XL,fsys_sectnr1
		rcall	fsys_wspi
		lds	XL,fsys_sectnr0
		rcall	fsys_wspi
fsys_swrite_1:	ldi	XL,0x95			;dummy crc
		rcall	fsys_wspi
		clr	r0
fsys_swrite_2:	rcall	fsys_write_ff
		cpi	XL,0xff
		brne	fsys_swrite_3
		dec	r0
		brne	fsys_swrite_2

fsys_swrite_3:	ldi	XL,0xfe			;start block token
		rcall	fsys_wspi
		ldi	XH,0x00
		ldi	YL,LOW(fsys_buffer)
		ldi	YH,HIGH(fsys_buffer)
fsys_swrite_4:	ld	XL,Y+
		rcall	fsys_wspi
		ld	XL,Y+
		rcall	fsys_wspi
		dec	XH
		brne	fsys_swrite_4
		rcall	fsys_write_ff2
fsas_swrite_5:	rcall	fsys_write_ff
		cpi	XL,0xff
		brne	fsas_swrite_5
		sbi	SPI_SPORT,SPI_SPIN	;de-activate
		ret

fsys_swrite_10:	lds	XL,fsys_sectnr2
		lds	XH,fsys_sectnr1
		lsl	XH
		rol	XL
		rcall	fsys_wspi
		mov	XL,XH
		lsr	XL
		lds	XH,fsys_sectnr0
		lsl	XH
		rol	XL
		rcall	fsys_wspi
		mov	XL,XH
		rcall	fsys_wspi
		clr	XL
		rcall	fsys_wspi
		rjmp	fsys_swrite_1


;-----------------------------------------------------------------------------
; read master boot record
;-----------------------------------------------------------------------------
fsys_rdmbr:	sts	fsys_sectnr3,const_0
		sts	fsys_sectnr2,const_0
		sts	fsys_sectnr1,const_0
		sts	fsys_sectnr0,const_0
		rcall	fsys_sread		;read sector into buffer
		lds	XL,fsys_buffer+0x1fe	;check MBR signature
		lds	XH,fsys_buffer+0x1ff
		cpi	XL,0x55
		brne	fsys_rdmbr_err
		cpi	XL,0x55
		breq	fsys_rdmbr_0
fsys_rdmbr_err:	ldi	r16,0x01		;NO MBR
		ret

fsys_rdmbr_0:	ldi	YL,LOW(fsys_buffer+0x1c2)
		ldi	YH,HIGH(fsys_buffer+0x1c2)
		ldi	XH,4
		clr	r16			;OK
fsys_rdmbr_1:	ld	XL,Y
		cpi	XL,0x06			;FAT16
		breq	fsys_rdmbr_3
		adiw	YL,16
		dec	XH
		brne	fsys_rdmbr_1
		ldi	r16,0x02		;NO FAT16 partition
fsys_rdmbr_2:	ret
		;set address of boot sector
fsys_rdmbr_3:	ldd	XH,Y+4
		sts	fsys_sectnr0,XH
		ldd	XH,Y+5
		sts	fsys_sectnr1,XH
		ldd	XH,Y+6
		sts	fsys_sectnr2,XH
		ldd	XH,Y+7
		sts	fsys_sectnr3,XH
		ret

;-----------------------------------------------------------------------------
; read boot sector
;-----------------------------------------------------------------------------
fsys_rbs:	rcall	fsys_sread		;read sector into buffer
		lds	XL,fsys_buffer+0x0b
		cpi	XL,0x00			;only 512 bytes/sector allowed
		brne	fsys_rbs_err
		lds	XL,fsys_buffer+0x0c
		cpi	XL,0x02			;only 512 bytes/sector allowed
		breq	fsys_rbs_1

fsys_rbs_err:	ldi	r16,0x03		;wrong geometry
		ret

fsys_rbs_1:	lds	XL,fsys_buffer+0x0e	;reserved sectors
		lds	XH,fsys_buffer+0x0f
		clr	YL
		clr	YH
		rcall	fsys_addsec		;add reserved sectors

		push	r16
		push	r17
		push	r18

		lds	r16,fsys_buffer+0x16	;sectors per FAT
		lds	r17,fsys_buffer+0x17
		lds	r18,fsys_buffer+0x10	;number of FAT copies

		mul	r16,r18
		movw	XL,r0
		clr	YL
		clr	YH
		mul	r17,r18
		add	XH,r0
		adc	YL,r1
		adc	YH,const_0
		pop	r18
		pop	r17
		pop	r16
		rcall	fsys_addsec		;add FAT sectors, we got the address of root directory
		ret

;-----------------------------------------------------------------------------
; read root directory
;-----------------------------------------------------------------------------
fsys_rdir:	rcall	fsys_sread		;read sector into buffer
		ldi	YL,LOW(fsys_buffer)
		ldi	YH,HIGH(fsys_buffer)
fsys_rdir_1:	ldi	ZL,LOW(fsys_dirent*2)
		ldi	ZH,HIGH(fsys_dirent*2)
		ld	XL,Y
		cpi	XL,0x00			;no entry
		breq	fsys_rdir_err
		push	YL
		push	YH
		ldi	XH,11			;bytes to compare
fsys_rdir_2:	elpm	r0,Z+			;get ROM data
		ld	r1,Y+			;get buffer data
		cp	r0,r1			;compare
		breq	fsys_rdir_3		;branch if equal
		pop	YH
		pop	YL
		adiw	YL,32			;next entry
		cpi	YH,HIGH(fsys_buffer)+2	;all done?
		brcs	fsys_rdir_1		;branch if not

fsys_rdir_err:	ldi	r16,0x04		;file not found
		ret

fsys_rdir_3:	dec	XH			;bytes counter
		brne	fsys_rdir_2		;next char
		pop	YH
		pop	YL

		push	r16
		push	r17
		push	r18

		ldd	r16,Y+0x1a		;offset
		ldd	r17,Y+0x1b
		ldi	r18,8

		mul	r16,r18
		movw	XL,r0
		clr	YL
		clr	YH
		mul	r17,r18
		add	XH,r0
		adc	YL,r1
		adc	YH,const_0
		ldi	r18,0x10
		add	XL,r18
		adc	XH,const_0
		adc	YL,const_0
		adc	YH,const_0
		pop	r18
		pop	r17
		pop	r16
		rcall	fsys_addsec		;add offset
;		rcall	fsys_viewoffset
		lds	XL,fsys_sectnr0
		sts	fsys_offset_l,XL
		lds	XL,fsys_sectnr1
		sts	fsys_offset_m,XL
		lds	XL,fsys_sectnr2
		sts	fsys_offset_h,XL
		lds	XL,fsys_sectnr3
		sts	fsys_offset_e,XL
		ret

fsys_dirent:	.db	"AX82BIMGBIN",0x00

;-----------------------------------------------------------------------------
; read file size
;-----------------------------------------------------------------------------
fsys_rsize:	rcall	fsys_sread		;read sector into buffer
		lds	XL,fsys_buffer+11
		ldi	XL,0x01			;offset=1 (head sector)
		ldi	XH,0x00
		ldi	YL,0x00
		ldi	YH,0x00

		rcall	fsys_addsec		;add offset
		ret

;-----------------------------------------------------------------------------
; select program
;-----------------------------------------------------------------------------
fsys_psel:	rcall	fsys_readscr

fsys_psel_1:	rcall	vsys_waitkey
		cpi	XL,0xea			;ENTER = OK
		brne	fsys_psel_2
		jmp	fsys_readfile

fsys_psel_2:	cpi	XL,0xe6			;PAGE UP -
		brne	fsys_psel_1
		lds	XL,fsys_filenr
		dec	XL
		andi	XL,0x0f
		sts	fsys_filenr,XL
		rjmp	fsys_psel

fsys_psel_3:	cpi	XL,0xe7			;PAGE DOWN +
		brne	fsys_psel_3
		lds	XL,fsys_filenr
		inc	XL
		andi	XL,0x0f
		sts	fsys_filenr,XL
		rjmp	fsys_psel


;------------------------------------------------------------------------------
; read name
; fsys_tapenr	= tape
; fsys_filenr	= file
;------------------------------------------------------------------------------
fsys_rname:	push	XL
		push	XH
		push	YL
		push	YH

		ldi	XL,0x7f			;last sector
		sts	fsys_sectnr,XL
		rcall	fsys_rsector
		ldi	YL,LOW(fsys_buffer)
		ldi	YH,HIGH(fsys_buffer)
		ldi	XH,13
fsys_rname_1:	ldd	XL,Y+35
		cpi	XL,0x20
		brcc	fsys_rname_2
		ldi	XL,0x20
fsys_rname_2:	st	Y+,XL
		dec	XH
		brne	fsys_rname_1
		st	Y+,const_0

		pop	YH
		pop	YL
		pop	XH
		pop	XL
		ret
;-----------------------------------------------------------------------------
; add sector offset
;-----------------------------------------------------------------------------
fsys_addsec:	push	r16
		lds	r16,fsys_sectnr0
		add	r16,XL
		sts	fsys_sectnr0,r16

		lds	r16,fsys_sectnr1
		adc	r16,XH
		sts	fsys_sectnr1,r16

		lds	r16,fsys_sectnr2
		adc	r16,YL
		sts	fsys_sectnr2,r16

		lds	r16,fsys_sectnr3
		adc	r16,YH
		sts	fsys_sectnr3,r16
		pop	r16
		ret


fsys_viewoffset1:
		push	r16
		lds	r16,fsys_offset_e
		call	sys_hexout
		lds	r16,fsys_offset_h
		call	sys_hexout
		lds	r16,fsys_offset_m
		call	sys_hexout
		lds	r16,fsys_offset_l
		call	sys_hexout
		pop	r16
		ret

fsys_viewoffset:
		push	r16
		lds	r16,fsys_sectnr3
		call	sys_hexout
		lds	r16,fsys_sectnr2
		call	sys_hexout
		lds	r16,fsys_sectnr1
		call	sys_hexout
		lds	r16,fsys_sectnr0
		call	sys_hexout
		pop	r16
		call	vsys_waitkey
		ret
