;################################################################################
;#										#
;# avr-chipbasic2 - single chip basic computer with ATmega644			#
;# EEPROM/Flash memory interface						#
;# copyright (c) 2006-2013 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

mem_spm:	in	r16,SPMCSR
		sbrc	r16,0
		rjmp	mem_spm
		cli
mem_spm1:	out	SPMCSR,XH
		spm
		sei
		ret

mem_waitrww:	in	r16,SPMCSR
		sbrs	r16,6
		ret
		ldi	XH,0x11
		rcall	mem_spm
		rjmp	mem_waitrww

;------------------------------------------------------------------------------
; save a snapshot to Flash
;------------------------------------------------------------------------------
mem_snap_usr:
		;copy the registers
		ldi	YH,HIGH(emu_memtab)
		ldi	YL,0

		st	Y+,PC_L
		st	Y+,PC_H

		st	Y+,SP_L
		st	Y+,SP_H

		st	Y+,reg_f
		st	Y+,reg_a

		st	Y+,reg_c
		st	Y+,reg_b
		st	Y+,reg_e
		st	Y+,reg_d
		st	Y+,reg_l
		st	Y+,reg_h

		in	XL,GPIOR0
		st	Y+,XL

		st	Y+,reg_border			;real border

		lds	XL,vsys_sborder			;spectrum border
		st	Y+,XL

		ldi	ZL,LOW(z80emu_ram)
		ldi	ZH,HIGH(z80emu_ram)
		ldi	XH,34
mem_save_sys_3:	ld	XL,Z+
		st	Y+,XL
		dec	XH
		brne	mem_save_sys_3
		ldi	r18,0xc0
		mov	reg_border,r18
		sbi	GPIOR0,noscreen_flag

		;now write the RAM to ROM
		movw	YL,const_0		;RAM addr
		ldi	ZL,0			;FLASH addr
		ldi	ZH,0
mem_wrom_1:	ldi	XH,0x03			;page erase
		rcall	mem_spm
		rcall	mem_waitrww
		ldi	XL,0x80			;no. of words to copy
mem_wrom_2:	ld	r0,Y+			;low byte
		ld	r1,Y+			;high byte
		ldi	XH,0x01			;write to buffer
		rcall	mem_spm
		adiw	ZL,2
		dec	XL
		brne	mem_wrom_2
		sbiw	ZL,2
		ldi	XH,0x05			;page write
		rcall	mem_spm
mem_wrom_3:	rcall	mem_waitrww
		adiw	ZL,2
		cpi	YH,max_pages+2		;end
		brcs	mem_wrom_1
		out	RAMPZ,const_1
;		sts	vsys_keycode,const_0
;		sts	vsys_lastkey,const_0
		lds	XL,vsys_wait
		subi	XL,50
mem_wrom_4:	lds	YL,vsys_wait
		cp	YL,XL
		brne	mem_wrom_4
		;verify
		sts	vsys_keycode,const_0
		sts	vsys_kbdstate,const_0
mem_wrom_5:	lds	XL,UCSR0A
		sbrs	XL,RXC0
		rjmp	mem_wrom_6
		lds	XL,UDR0
		rjmp	mem_wrom_5
mem_wrom_6:	cbi	GPIOR0,noscreen_flag
		movw	YL,const_0		;RAM addr
		ldi	ZL,0			;FLASH addr
		ldi	ZH,0
mem_wrom_v1:	ld	XL,Y+
		elpm	XH,Z+
		cp	XL,XH
		brne	mem_wrom_v2
		cpi	YH,max_pages+2		;end
		brcs	mem_wrom_v1
		jmp	emu_start_usr		;reload the current setting

mem_wrom_v2:	ldi	XL,0x40
		mov	reg_border,XL
		sts	vsys_xpos,const_0
		sts	vsys_ypos,const_0
		ldi	ZL,LOW(mem_etext*2)
		ldi	ZH,HIGH(mem_etext*2)
		call	sys_text
mem_wrom_v3:	rjmp	mem_wrom_v3

mem_etext:	.db	" !!! SNAPSHOT VERIFY ERROR !!!  ",0,0

		;write a single block to flash
mem_writeblock:	ldi	YH,HIGH(fsys_buffer)	;RAM addr
		ldi	YL,LOW(fsys_buffer)
		lds	ZL,fsys_mwp_l
		lds	ZH,fsys_mwp_h
		out	RAMPZ,const_1
		cpi	ZH,0x40
		brcs	mem_wblk_1
		out	RAMPZ,const_0

mem_wblk_1:	ldi	XH,0x03			;page erase
		rcall	mem_spm
		rcall	mem_waitrww

		ldi	XL,0x80			;no. of words to copy
mem_wblk_2:	ld	r0,Y+			;low byte
		ld	r1,Y+			;high byte
		ldi	XH,0x01			;write to buffer
		rcall	mem_spm
		adiw	ZL,2
		dec	XL
		brne	mem_wblk_2
		sbiw	ZL,2

		ldi	XH,0x05			;page write
		rcall	mem_spm
mem_wblk_3:	rcall	mem_waitrww
		adiw	ZL,2
		cpi	YH,HIGH(fsys_buffer)+2	;end
		brcs	mem_wblk_1
		out	RAMPZ,const_1
		ret

