;################################################################################
;#										#
;# vsys - multi i/o for ATMega644						#
;# main library									#
;# copyright (c) 2005-2010 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-------------------------------------------------------------------------------
;video-interrupt (no registers destroyed, stack=13)
;save registers (25c)
;-------------------------------------------------------------------------------
vint_vstart:	in	sreg_save,SREG		;1
		push	ZH			;2 save Z-register
		push	ZL			;2 save Z-register
		push	YH			;2 save Y-register
		push	YL			;2 save Y-register
		push	XH			;2 save X-register
		push	XL			;2 save X-register

;-------------------------------------------------------------------------------
;count line up
;-------------------------------------------------------------------------------
		add	vline_l,const_1		;1
		adc	vline_h,const_0		;1

;-------------------------------------------------------------------------------
;check if line is image
;-------------------------------------------------------------------------------
vint_tv:	movw	XL,vline_l
		sbrc	vline_h,0		;1 skip if less than 256
		rjmp	vint_tv_01		;2 no visible line
		cpi	XL,192			;1 max lines
		brcs	vint_tv_10

;-------------------------------------------------------------------------------
;no image line
;-------------------------------------------------------------------------------
vint_tv_01:	cpi	XL,vsys_vstart_t
		brne	vint_tv_02		;no vsync start
		cbi	VSPORT,VSPIN
		ldi	XL,0x82
		sts	TCCR1A,XL
		rjmp	vint_tv_end

vint_tv_02:	cpi	XL,vsys_vstop_t
		brne	vint_tv_03		;no vsync stop
		ldi	XL,0xc2
		sts	TCCR1A,XL
		sbi	VSPORT,VSPIN
		rjmp	vint_tv_end

vint_tv_03:	cpi	XL,vsys_vbstart_t
		brne	vint_tv_04		;no border start
		ldi	XL,0x05
		sts	TIMSK2,XL
		rjmp	vint_tv_end

vint_tv_04:	cpi	XL,vsys_vbstop_t
		brne	vint_tv_05		;no border stop
		sts	TIMSK2,const_1		;only border off interrupt
		rjmp	vint_tv_end

vint_tv_05:	cpi	XL,57
		breq	vint_tv_06		;2 end of frame
		rjmp	vint_tv_end

vint_tv_06:	jmp	vint_frame

;-------------------------------------------------------------------------------
;sync to counter
;-------------------------------------------------------------------------------
vint_tv_10:;	rjmp	vint_tv_end

vint_tv0_sy0:	lds	XL,TCNT1L		;1 get timer
		cpi	XL,vsys_hstart_t	;1
		brcs	vint_tv0_sy0		;1 loop
vint_tv0_sy_01:	cpi	XL,vsys_hstart_t+1
		brcs	vint_tv0_sy_02
vint_tv0_sy_02:	cpi	XL,vsys_hstart_t+2
		brcs	vint_tv0_sy_03
vint_tv0_sy_03:	cpi	XL,vsys_hstart_t+3
		brcs	vint_tv0_sy_04
vint_tv0_sy_04:	cpi	XL,vsys_hstart_t+4
		brcs	vint_tv0_sy_05
vint_tv0_sy_05:

		sbic	GPIOR0,noscreen_flag	;skip if screen enabled
		rjmp	vint_ls_end

vint_tv_vout:	push	r16
		push	r17
		push	r18
		push	r19
		push	r20
		push	r21
		push	r22
		push	r0
		push	r1

		ldi	r22,26			;words per 8 pix shift
		lds	r21,vsys_blinkt

		ldi	XL,3			;3 entries per line
		mul	XL,vline_l

		ldi	ZH,HIGH(vad_table*2)
		ldi	ZL,LOW(vad_table*2)
		add	ZL,r0
		adc	ZH,r1
		elpm	XL,Z+
		elpm	XH,Z+
		elpm	YH,Z+
		mov	YL,XL
		ldi	r20,32			;bytes to do

		ld	ZL,Y+			;get attr
		mov	ZH,r21
		elpm	r16,Z
		mov	r17,r16
		swap	r16
		ld	ZL,X+			;pix
		mul	ZL,r22
		movw	ZL,r0
		subi	ZH,0x60			;+0xa000
		ijmp

vint_end:	pop	r1
		pop	r0
		pop	r22
		pop	r21
		pop	r20
		pop	r19
		pop	r18
		pop	r17
		pop	r16
		sei

vint_tv_22:	mov	YH,vline_l		;
		andi	YH,0x03
		breq	vint_tv_end

vint_iend:	pop	XL			;2 restore Y-register
		pop	XH			;2 restore Y-register
		pop	YL			;2 restore Y-register
		pop	YH			;2 restore Y-register
		pop	ZL			;2 restore Z-register
		pop	ZH			;2 restore Z-register
		out	SREG,sreg_save		;1
		reti				;4 thats all

vint_ls_end:	ldi	XL,0xc0
		sbrc	vline_l,2
		ldi	XL,0x20
		mov	reg_border,XL
		rjmp	vint_iend

vint_tv_end:	sbic	GPIOR0,noscreen_flag	;skip if screen enabled
		rjmp	vint_ls_end
